"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTime = void 0;
const dgram_1 = require("dgram");
const TRANSMIT_TIME_OFFSET = 40;
const PACKET_SIZE = 48;
const TIME_REQUEST = 0x11;
const TIME_LOCAL_BASE = new Date(0, 0).getTime();
const TIME_UTC_BASE = Date.UTC(0, 0);
/**
 *
 *
 * @export
 * @param {GetTimeOptions} [options={}]
 * @param {string} [options.server="pool.ntp.org"] NTP server address
 * @param {number} [options.port=123] NTP server port
 * @param {number} [options.timeout=5000] Timeout in milliseconds
 * @param {SocketType} [options.socketType="udp4"] UDP Socket type (IP)
 * @param {boolean} [options.utc=true] Whether use UTC or Local time
 * @return {Promise<Date>} Synced date
 */
async function getTime({ server = "pool.ntp.org", port = 123, timeout = 5000, socketType = "udp4", utc = true, } = {}) {
    const socket = (0, dgram_1.createSocket)(socketType);
    const packet = Buffer.alloc(PACKET_SIZE).fill(0);
    packet[0] = TIME_REQUEST;
    let onTimeout = () => { };
    const response = new Promise((resolve, reject) => {
        onTimeout = () => reject(new Error("Timeout exceeded"));
        socket
            .once("error", reject)
            .once("message", resolve)
            .send(packet, port, server, (error) => {
            if (error !== null)
                reject(error);
        });
    });
    const deadline = setTimeout(() => onTimeout(), timeout);
    const buffer = await response.finally(() => {
        clearInterval(deadline);
        socket.close();
    });
    const seconds = buffer.readUInt32BE(TRANSMIT_TIME_OFFSET);
    const base = utc ? TIME_UTC_BASE : TIME_LOCAL_BASE;
    return new Date(base + seconds * 1000);
}
exports.getTime = getTime;
